/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.openiec61850.common.model.datypes.basic;

import org.openiec61850.common.model.BasicDataAttribute;
import org.openiec61850.common.model.FunctionalConstraint;
import org.openiec61850.common.model.ObjectReference;
import org.openiec61850.common.model.datypes.DAType;

public class UNICODE_STRING extends BasicDataAttribute {

	private String value;
	private int maxLength;

	public UNICODE_STRING(ObjectReference objectReference, FunctionalConstraint fc, String sAddr, String value,
			int maxLength, boolean dchg, boolean dupd) {
		if (value.length() > maxLength) {
			throw new IllegalArgumentException("max_length less than value.length().");
		}
		this.objectReference = objectReference;
		this.fc = fc;
		this.basicType = DAType.UNICODE_STRING;
		this.sAddr = sAddr;
		this.value = value;
		this.dchg = dchg;
		this.dupd = dupd;
		this.maxLength = maxLength;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		if (value.length() > maxLength) {
			throw new IllegalArgumentException("max_length less than value.length().");
		}
		this.value = value;
	}

	public int getMaxLength() {
		return maxLength;
	}

	@Override
	public void setDefault() {
		value = "";
	}

	@Override
	public UNICODE_STRING copy() {
		return new UNICODE_STRING(objectReference, fc, sAddr, value, maxLength, dchg, dupd);
	}

}
